@@ -8,12 +8,15 @@ sub tree_ {
     'assertion_call' => {
       '.any' => [
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_eq'
         },
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_ok'
         },
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_has'
         }
       ]
@@ -24,9 +27,11 @@ sub tree_ {
     'assertion_eq' => {
       '.any' => [
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_operator_eq'
         },
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_function_eq'
         }
       ]
@@ -63,9 +68,11 @@ sub tree_ {
     'assertion_has' => {
       '.any' => [
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_operator_has'
         },
         {
+          '-wrap' => 1,
           '.ref' => 'assertion_function_has'
         }
       ]
@@ -118,7 +125,7 @@ sub tree_ {
           '.ref' => 'block_marker'
         },
         {
-          '+qty' => '?',
+          '+max' => 1,
           '.all' => [
             {
               '.rgx' => qr/(?-xism:\G[\ \t]+)/
@@ -158,8 +165,7 @@ sub tree_ {
           '.ref' => 'code_object'
         },
         {
-          '+qty' => '*',
-          '-pass' => 1,
+          '+min' => 0,
           '.ref' => 'unit_call'
         }
       ]
@@ -184,7 +190,7 @@ sub tree_ {
       ]
     },
     'code_section' => {
-      '+qty' => '*',
+      '+min' => 0,
       '.any' => [
         {
           '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)+)/
@@ -203,7 +209,7 @@ sub tree_ {
           '.ref' => 'code_expression'
         },
         {
-          '+qty' => '?',
+          '+max' => 1,
           '.ref' => 'assertion_call'
         },
         {
@@ -223,7 +229,7 @@ sub tree_ {
           '.ref' => 'block_header'
         },
         {
-          '+qty' => '*',
+          '+min' => 0,
           '-skip' => 1,
           '.any' => [
             {
@@ -235,14 +241,13 @@ sub tree_ {
           ]
         },
         {
-          '+qty' => '*',
-          '-pass' => 1,
+          '+min' => 0,
           '.ref' => 'block_point'
         }
       ]
     },
     'data_section' => {
-      '+qty' => '*',
+      '+min' => 0,
       '.ref' => 'data_block'
     },
     'double_quoted_string' => {
@@ -251,15 +256,14 @@ sub tree_ {
     'function_object' => {
       '.all' => [
         {
-          '+qty' => '?',
-          '-pass' => 1,
+          '+max' => 1,
           '.ref' => 'function_signature'
         },
         {
           '.ref' => 'function_start'
         },
         {
-          '+qty' => '*',
+          '+min' => 0,
           '.any' => [
             {
               '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)+)/
@@ -283,7 +287,7 @@ sub tree_ {
           '.rgx' => qr/(?-xism:\G\((?:[\ \t]|\r?\n|\#.*\r?\n)*)/
         },
         {
-          '+qty' => '?',
+          '+max' => 1,
           '.ref' => 'function_variables'
         },
         {
@@ -298,22 +302,11 @@ sub tree_ {
       '.rgx' => qr/(?-xism:\G([a-zA-Z]\w*))/
     },
     'function_variables' => {
-      '.all' => [
-        {
-          '.ref' => 'function_variable'
-        },
-        {
-          '+qty' => '*',
-          '.all' => [
-            {
-              '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)*,(?:[\ \t]|\r?\n|\#.*\r?\n)*)/
-            },
-            {
-              '.ref' => 'function_variable'
-            }
-          ]
-        }
-      ]
+      '+min' => 1,
+      '.ref' => 'function_variable',
+      '.sep' => {
+        '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)*,(?:[\ \t]|\r?\n|\#.*\r?\n)*)/
+      }
     },
     'lines_point' => {
       '.all' => [
@@ -378,17 +371,14 @@ sub tree_ {
       '.rgx' => qr/(?-xism:\G(\*[a-z]\w*))/
     },
     'point_phrase' => {
-      '-pass' => 1,
       '.ref' => 'unquoted_string'
     },
     'quoted_string' => {
       '.any' => [
         {
-          '-pass' => 1,
           '.ref' => 'single_quoted_string'
         },
         {
-          '-pass' => 1,
           '.ref' => 'double_quoted_string'
         }
       ]
@@ -415,7 +405,7 @@ sub tree_ {
           '.ref' => 'code_section'
         },
         {
-          '+qty' => '?',
+          '+max' => 1,
           '.ref' => 'data_section'
         }
       ]
@@ -429,33 +419,17 @@ sub tree_ {
           '.rgx' => qr/(?-xism:\G\((?:[\ \t]|\r?\n|\#.*\r?\n)*)/
         },
         {
-          '+qty' => '?',
-          '-pass' => 1,
-          '.ref' => 'transform_arguments'
+          '+min' => 0,
+          '.ref' => 'transform_argument',
+          '.sep' => {
+            '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)*,(?:[\ \t]|\r?\n|\#.*\r?\n)*)/
+          }
         },
         {
           '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)*\))/
         }
       ]
     },
-    'transform_arguments' => {
-      '.all' => [
-        {
-          '.ref' => 'transform_argument'
-        },
-        {
-          '+qty' => '*',
-          '.all' => [
-            {
-              '.rgx' => qr/(?-xism:\G(?:[\ \t]|\r?\n|\#.*\r?\n)*,(?:[\ \t]|\r?\n|\#.*\r?\n)*)/
-            },
-            {
-              '.ref' => 'transform_argument'
-            }
-          ]
-        }
-      ]
-    },
     'transform_name' => {
       '.any' => [
         {
@@ -472,7 +446,7 @@ sub tree_ {
           '.ref' => 'transform_name'
         },
         {
-          '+qty' => '?',
+          '+max' => 1,
           '.ref' => 'transform_argument_list'
         }
       ]
